// import { generateAxios } from '@/config/http.js';
// import { Message } from 'element-ui';


// // 构造axios
// const axios = generateAxios({});
// 最大并发数
const MAX_POOL = 5;

// 切片的默认大小(S3的切片最小值为5M)
const DefaultChunkSize = 5 * 1024 * 1024;


// 生成promise
const createPromise = function (index, chunkBlobArray, chunkParams) {
    return new Promise((resolve, reject)=>{
        this.$generateAxios({
            method: 'post',
            url: '/doc-front-end/document/chunkDownload',
            data: chunkParams,
            responseType: 'blob',
        }).then(res=>{
            chunkBlobArray[index] = res;
            resolve();
        }).catch(()=>{
            reject();
        });
    });
};

export const chunkFileDownload = async function (
    fileInstance,
) {
    // 并发池
    const runningPromises = [];
    try {
        const {
            fileName,
            fileSize,
            fileId
        } = fileInstance;
        // 通过接口获取到当前文件是在桶的哪个位置？
        const s3ConfigRes = await this.$generateAxios({
            method: 'get',
            url: `/doc-front-end/document/getFileProperty/${fileId}`,
        });
        // 如果接口响应是错误的，直接抛出异常
        if (s3ConfigRes.code !== 200) {
            throw new Error(s3ConfigRes.msg);
        }
        // 否则去拿出owner 和 bucket
        const {
            owner,
            bucket
        } = s3ConfigRes.data;
        // 一共有多少个分片，去设置一个全为null的数据
        const chunkNumber = Math.ceil(fileSize / DefaultChunkSize);
        // 根据分片数量创建一个存储分片blob流的数组
        const chunkBlobArray = Array(chunkNumber).fill(null);
        // 平分每一个分片的大小
        const chunkSize = Math.ceil(fileSize / chunkNumber);
        // 构建所有请求
        const requestList = chunkBlobArray.map((chunkBlobBox, index)=>{
            return ()=>{
                const chunkParams = {
                    fileId,
                    startByte: chunkSize * index,
                    endByte: (i + 1) * chunkSize - 1,
                    chunkNum: index + 1,
                    fileName,
                    owner,
                    bucket
                };
                return createPromise.call(this, index, chunkBlobArray, chunkParams);
            };
        });
        let i = 0;
        while (i < requestList.length) {
            const promiseFn = requestList[i];
            const promise = (await promiseFn)();
            runningPromises.push(promise);
            i++;
            if (runningPromises.length >= MAX_POOL) {
                await Promise.race(runningPromises); // 等待任一 Promise 完成
                // 移除已完成的 Promise
                const completedPromise = runningPromises.shift();
                await completedPromise;
            }
        }
        await Promise.all(runningPromises); // 等待剩余的 Promise 全部完成
        const blob = new Blob(chunkBlobArray, { type: 'application/octet-stream' });
        // webkix兼容性处理
        const URL = window.URL || window.webkitURL;
        const link = document.createElement('a');
        link.href = URL.createObjectURL(blob);
        link.download = fileName;
        document.body.appendChild(link);
        link.click();
        // 在dom中移除a标签
        document.body.removeChild(link);
        // 在内存中移除URL对象
        window.URL.revokeObjectURL(URL);
        return {
            code: 200,
        };
    } catch (error) {
        const errorMessage = error && error.message ? error.message : '网络异常，请稍后再试！';
        this.$message.error(errorMessage);
        return {
            code: 500,
        };
    }
};
