import CryptoJS from 'crypto-js';
// import { generateAxios } from '@/config/http.js';
import { Message } from 'element-ui';


// // 构造axios
// const axios = generateAxios({});
// 最大并发数
const MAX_POOL = 5;


// 切片的默认大小(S3的切片最小值为5M)
const DefaultChunkSize = 5 * 1024 * 1024;

// 对文件进行切片
function createFileChunk (file) {
    return new Promise((resolve, reject) => {
        const fileSize = file.size;
        const chunks = Math.ceil(fileSize / DefaultChunkSize);
        const result = [];
        const reader = new FileReader();

        reader.onloadend = () => {
            if (reader.error) {
                reject(reader.error);
                return;
            }
            const chunk = new Blob([ reader.result ]);
            result.push(chunk);

            if (result.length === chunks || chunk.size > DefaultChunkSize || chunk.size < DefaultChunkSize) {
                resolve(result);
            } else {
                loadNextChunk();
            }
        };

        function loadNextChunk () {
            const start = result.length * DefaultChunkSize;
            const end = Math.min(start + DefaultChunkSize, fileSize);
            let chunk;
            if (result.length === chunks - 2 && fileSize % DefaultChunkSize !== 0) {
                chunk = file.slice(start);
            } else {
                chunk = file.slice(start, end);
            }
            reader.readAsArrayBuffer(chunk);
        }

        loadNextChunk();
    });
}

// // 生成MD5文件
// const calculateMD5 = (file)=> {
//     return new Promise((resolve, reject) => {
//         const reader = new FileReader();
//         reader.onload = (event) => {
//             const arrayBuffer = event.target.result;
//             const md5sum = CryptoJS.MD5(CryptoJS.lib.WordArray.create(arrayBuffer));
//             resolve(md5sum.toString());
//         };
//
//         reader.onerror = (event) => {
//             reject(event.target.error);
//         };
//
//         reader.readAsArrayBuffer(file);
//     });
// }

// // 对切片进行MD5加密
// async function encryptFile(file) {
//     return new Promise((resolve, reject) => {
//
//         const reader = new FileReader();
//         let offset = 0;
//         const hasher = CryptoJS.algo.MD5.create();
//
//         reader.onload = () => {
//             const wordArray = CryptoJS.lib.WordArray.create(reader.result);
//             hasher.update(wordArray); // 更新哈希计算
//             offset += wordArray.sigBytes;
//
//             if (offset < file.size) {
//                 readNextChunk();
//             } else {
//                 const hash = hasher.finalize().toString();
//                 resolve(hash);
//             }
//         };
//
//         reader.onerror = () => {
//             reject(new Error('文件读取错误'));
//         };
//
//         function readNextChunk() {
//             const blob = file.slice(offset, offset + DefaultChunkSize);
//             reader.readAsArrayBuffer(blob);
//         }
//
//         readNextChunk();
//     });
// }




// 生成promise
const createPromise = function (totalSum, fileChunk, chunkUploadParams, fileName, chunkIndex, updateProgress, owner, bucket, chunksProgressArray) {
    const md5Key = CryptoJS.MD5(`${fileName}${chunkIndex}${fileChunk.size}`).toString();
    // 参数
    const formData = new FormData();
    formData.append('uploadId', chunkUploadParams.uploadId);
    formData.append('s3Key', chunkUploadParams.s3Key);
    formData.append('filename', fileName);
    formData.append('file', fileChunk); // 文件
    formData.append('chunkNumber', chunkIndex + 1); // 当前切片的下标
    formData.append('chunkSize', fileChunk.size); // 当前切片的大小
    formData.append('chunkMd5Key', md5Key);
    formData.append('owner', owner); // 上传到哪里
    formData.append('bucket', bucket); // 桶名
    return new Promise((resolve, reject)=> {
        this.$generateAxios({
            method: 'post',
            url: '/doc-front-end/document/uploadChunk',
            data: formData,
            headers: {
                'Content-Type': 'application/form-data',
            },
            timeout: 600000,
            onUploadProgress (progress) {
                if (progress.total && updateProgress) {
                    chunksProgressArray[chunkIndex] = progress.loaded / totalSum * 100;
                    const progressTotal = chunksProgressArray.reduce((accumulator, currentValue) => {
                        return accumulator + currentValue;
                    }, 0);
                    // 做个保护：
                    if (progressTotal > 100) {
                        updateProgress(100);
                    } else {
                        updateProgress(progressTotal);
                    }
                }
            },
        }).then(res=>{
            if (res.code === 200) {
                resolve();
            } else {
                reject();
            }
        }).catch(()=>{
            reject();
        });
    });
};


/**
 * file：element直接拿到的file对象（必填）
 * handleThen：合并成功的处理方法（必填）
 * handleCatch：如果某一步发生了错误的处理方法（必填）
 * businessParams：业务参数（非必填，默认空对象）
 * updateProgress：更新当前上传进度（非必填，默认null）
 * topDirectory：S3顶级目录【非必填，默认wti-document（一体化），可选值：wti-consignment（代销）、wti-contract（合同）、wti-bill（票据）】
 * owner：上传到哪里【非必填，默认YTH（一体化）、DZHT（电子合同）】
 * bucket：桶名【非必填，默认pub（共有）、prv（私有）】
 *
 * */
export const runPromisesWithConcurrencyLimit = async function (
    file,
    handleThen,
    handleCatch,
    businessParams = {},
    updateProgress = null,
    s3Config = {},
){
    // 并发池
    const runningPromises = [];
    // 保存当前所有进度的数据
    const chunksProgressArray = [];
    try {
        const {
            topDirectory = 'wti-document',
            owner = 'YTH',
            bucket = 'pub',
        } = s3Config;
        // console.log(Date.now(), '开始', file.file);
        // 对文件进行切片
        const fileChunkList = await createFileChunk(file.file);
        // 获得经过MD5加密后的值
        const md5Key = CryptoJS.MD5(`${file.file.name}${file.file.size}`).toString();

        let type = '';
        if (file.file.type.includes('image')) {
            type = file.file.type;
        } else {
            type = 'file';
        }
        // 发起切片请求的接口参数
        const uploadRequestParams = {
            fileName: file.file.name,
            type,
            size: file.file.size,
            topDirectory: topDirectory,
            totalChunks: fileChunkList.length,
            fileKey: md5Key, // md5加密文件
            owner: owner,
            bucket: bucket,
            ...businessParams,
        };
        // 发起切片请求
        const chunkUploadParams = await this.$generateAxios({
            method: 'post',
            url: '/doc-front-end/document/uploadRequest',
            data: uploadRequestParams,
        });
        if (chunkUploadParams.code !== 200) {
            throw new Error(chunkUploadParams.msg);
        //  如果返回的信息中有fileId，那么说明之前已经上传过这个文件了，后端会直接挂关系
        } else if (chunkUploadParams.data.fileId) {
            handleThen(chunkUploadParams);
            return;
        }
        // 构建所有请求
        const requestList = fileChunkList.map((fileChunk, index) => () => createPromise.call(this, file.file.size, fileChunk, chunkUploadParams.data, file.file.name, index, updateProgress, owner, bucket, chunksProgressArray));

        let i = 0;
        while (i < requestList.length) {
            const promiseFn = requestList[i];
            const promise = (await promiseFn)();

            runningPromises.push(promise);
            i++;

            if (runningPromises.length >= MAX_POOL) {
                await Promise.race(runningPromises); // 等待任一 Promise 完成

                // 移除已完成的 Promise
                const completedPromise = runningPromises.shift();
                await completedPromise;
            }
        }
        await Promise.all(runningPromises); // 等待剩余的 Promise 全部完成
        // 合并请求的参数
        const mergeFileParams = {
            uploadId: chunkUploadParams.data.uploadId,
            s3Key: chunkUploadParams.data.s3Key,
            owner: owner,
            bucket: bucket,
            fileName:file.file.name,
            fileKey: md5Key,
            type: file.file.type,
            size: file.file.size,
            ...businessParams,
        };
        // 发起切片合并请求
        const mergeFileRes = await this.$generateAxios({
            method: 'post',
            url: '/doc-front-end/document/uploadMerge',
            data: mergeFileParams,
        });
        if (mergeFileRes.code === 200) {
            handleThen(mergeFileRes);
            // console.log(Date.now(), '结束');
            // 删除数据库存储的切片数据
            this.$generateAxios({
                method: 'get',
                url: `/doc-front-end/document/deleteChunk/${chunkUploadParams.data.uploadId}`,
            });
        } else {
            throw new Error(mergeFileRes.msg);
        }
    } catch (error) {
        const errorMessage = error && error.message ? error.message : '网络异常，请稍后再试！';
        Message({
            message: errorMessage,
            type: 'error',
        });
        // 做个保护
        if (handleCatch) {
            handleCatch();
        }
    }
};
