import axios from 'axios';
import ClassificationUploadFile from './index.vue';
import {Message} from 'element-ui';

const getFromSearchURL = (k) => {
    let v = null;
    const searchStr = window.location.search;
    if (searchStr.length > 1) {
        // 先拿到kv数组
        const searchList = searchStr.slice(1).split('&');
        searchList.forEach(kv => {
            const kvList = kv.split('=');
            if (kvList[0] === decodeURIComponent(k)) {
                v = decodeURIComponent(kvList[1]);
            }
        });
    }
    return v;
};
const constructionHTTPFun = (tokenField, baseURL) => {
    class ConstructionHTTP {
        constructor (baseURL) {
            axios.defaults.baseURL = baseURL;
        }

        requestSuccess (config) {
            let token = '';
            if (sessionStorage.getItem(tokenField)) {
                token = sessionStorage.getItem(tokenField);
                token = token ? '' + token : '';
            } else {
                token = getFromSearchURL('token');
                token = token ? '' + token : '';
            }
            config.headers.Authorization = token;
            return config;
        }

        requestFault (err) {
            return Promise.reject(err);
        }

        responseSuccess (response) {
            if (response.data.code === 401 && window.isMessageREpate) {
                window.isMessageREpate = false;
                // eslint-disable-next-line
                Message({
                    message: '登陆认证已过期,请重新登陆！',
                    type: 'warning'
                });
                sessionStorage.removeItem(tokenField);
                top.location.href = top.location.origin + '/login';
                // window.top.location.href = '/login.html';
            } else if (response.data.code !== 200 && response.data.code !== 201) {
                window.top.postMessage(
                    {
                        message: response.data.msg,
                        type: 'warning',
                        wti_type: 'message'
                    },
                    '*'
                );
            }
            return response.data;
        }

        responseFault (error) {
            window.top.postMessage(
                {
                    message: '您的网络连接失败，请稍后再试',
                    type: 'warning',
                    wti_type: 'message'
                },
                '*'
            );
            return Promise.reject(error.response.data);
        }

        generateAxios (config) {
            const newAxios = axios.create(config);
            newAxios.interceptors.request.use(this.requestSuccess, this.requestFault);
            newAxios.interceptors.response.use(this.responseSuccess, this.responseFault);
            return newAxios;
        }
    }
    const HTTP = new ConstructionHTTP(baseURL);
    const post = (url, data, params) => {
        const myAxios = HTTP.generateAxios();
        return myAxios({
            method: 'post',
            url,
            params,
            data,
            timeout: 1000 * 30,
        });
    };
    const get = (url, params, data) => {
        const myAxios = HTTP.generateAxios();
        return myAxios({
            method: 'get',
            url,
            data,
            params,
            timeout: 1000 * 30
        });
    };

    const documentFlow = (url, data, responseType) => {
        const myAxios = HTTP.generateAxios();
        return myAxios({
            method: 'post',
            url,
            data,
            responseType,
            timeout: 1000 * 600
        });
    };
    const generateAxios = (config) => {
        const myAxios = HTTP.generateAxios();
        return myAxios(config);
    };
    return {post, get, generateAxios, documentFlow};
};

export default {
    install: (Vue, installOptions = {}) => {
        // Object.assign(ClassificationUploadFile.props, installOptions);
        const {
            post,
            get,
            generateAxios,
            documentFlow
        } = constructionHTTPFun(installOptions.tokenField, installOptions.baseURL);
        Vue.prototype.$ajax = Object.assign(Vue.prototype.$ajax, {
            // 获取模板结构数据
            getDocFrameworkData (payload) {
                return post('/project-front-end/document/getInitData', payload);
            },
            // 获取结构对应的数据（已完成）
            getDocFrameworkDataFinish (payload) {
                return post('/project-front-end/document/query_upload_file', payload);
            },
            // 查分类名
            getCategoryCode2Name (payload) {
                return get(`/wkbbackend/dictCodeList/${payload.categoryCode}/${payload.dictCode}`);
            },
            // 获取模板结构
            getDocFramework (payload) {
                return post('/project-front-end/document/getDocFramework', payload);
            },
            // 批量删除文件
            deleteAllFile (payload) {
                return post('/project-front-end/document/deleteAll', payload);
            },
            downloadFileStreamXls (payload, responseType) {
                return documentFlow('/project-front-end/document/downloadFileStreamv4', payload, responseType);
            },
            loadDict (payload) {
                return post('/wkbbackend/queryByCategoryCodeList', payload);
            },
            // 查詢父級待辦
            getParentTaskInfo (payload) {
                return get(`/project-front-end/workflow/queryTaskInfoByOriginKey/${payload}`);
            },
            batchDownloadFiles (payload, responseType) {
                return documentFlow('/project-front-end/document/documentInfo/downloadFile', payload, responseType);
            },
            // 删除文件
            documentInfodelete (payload) {
                return get('/project-front-end/document/documentInfo/delete', payload);
            },
            // 查文件数据
            docInfoQueryByOriginalKeyNew (payload) {
                return post('/project-front-end/component/docInfoQueryByOriginalKeyNew', payload);
            },
            // 新删除文件
            docDeleteUploadFile (payload) {
                return post('/doc-front-end/document/deleteFile', payload);
            },
        });
        Vue.prototype.$classificationUploadFileFromCode = installOptions.fromCode || '';
        Vue.prototype.$generateAxios = generateAxios;
        Vue.component(ClassificationUploadFile.name, ClassificationUploadFile);
    }
};
