import { chunkFileDownload } from './chunkFileDownload';
export default {
    methods: {
        // 预览的统一方法
        handleFileReview (fileItem) {
            const typeName = fileItem.fileName ? fileItem.fileName.split('.') : fileItem.name.split('.');
            const fileType = typeName[typeName.length - 1];
            const u = this.makeOnlySearchURL({
                fileUrl: fileItem.attachmentUrl || fileItem.fileUrl || fileItem.filePath || fileItem.path,
                fileType,
                fileName: fileItem.fileName || fileItem.name,
            });
            this.openPage(404, u);
        },
        // URL 参数处理
        makeOnlySearchURL (kvObj) {
            // 获取一个完整的对象，如果用到同 k，用 kvObj 覆盖他
            const resultSearchUrl = Object.keys(kvObj).map(key => {
                return `${encodeURIComponent(key)}=${encodeURIComponent(
                    kvObj[key])}`;
            }).join('&');

            return `${resultSearchUrl}`;
        },
        // 打开页面
        openPage (id, searchUrl) {
            window.top.postMessage({
                id,
                url: searchUrl,
                type: 'open',
            }, '*');
        },
        // 切片下载单个文件
        chunkDownloadOneFile (item) {
            if (item.downloadFlag) {
                return;
            }
            this.$set(item, 'downloadFlag', true);
            item.downloadFlag = true;
            chunkFileDownload.call(this, {
                fileName: item.fileName || item.name,
                fileSize: item.size,
                fileId: item.fileId,
            }).then(()=>{
                this.$set(item, 'downloadFlag', false);
            });
        },
        // 单个文件下载
        downloadOneFile (item) {
            if (item.downloadFlag) {
                return;
            }
            this.$set(item, 'downloadFlag', true);
            item.downloadFlag = true;
            const typeName = item.fileName ? item.fileName.split('.') : item.name.split('.');
            const fileType = typeName[typeName.length - 1];
            this.$ajax.downloadFileStreamXls({
                url: item.attachmentUrl || item.fileUrl || item.filePath || item.path,
                name: item.fileName || item.name,
                type: fileType,
            }, 'blob').then(res => {
                const fileTypeMap = {
                    pdf: 'application/pdf;charset=UTF-8',
                    dotx: 'application/msword;charset=UTF-8',
                    docx: 'application/msword;charset=UTF-8',
                    doc: 'application/msword;charset=UTF-8',
                    mp4: 'audio/mp4;charset=UTF-8',
                    wav: 'audio/mav;charset=UTF-8',
                    wma: 'audio/wma;charset=UTF-8',
                    mp3: 'audio/mp3;charset=UTF-8',
                    txt: 'text/plain;charset=UTF-8',
                    md: 'text/plain;charset=UTF-8',
                    json: 'text/plain;charset=UTF-8',
                    csv: 'text/plain;charset=UTF-8',
                    zip: 'application/zip;charset=UTF-8',
                    rar: 'application/zip;charset=UTF-8',
                    jpg: 'application/x-jpg;charset=UTF-8',
                    png: 'application/x-png;charset=UTF-8',
                    jpeg: 'text/plain;charset=UTF-8',
                };
                // 创建blob对象，利用Blob解析流数据
                const blob = new Blob([ res ], {
                    type: fileTypeMap[fileType],
                });
                // 根据解析后的blob对象创建URL对象即下载链接
                const fileName = item.fileName ? item.fileName : item.name;
                this.handleDownload(blob, fileName);
            }).catch(() => {
                this.$message.error('资源下载失败，请稍后重试');
            }).finally(()=>{
                this.$set(item, 'downloadFlag', false);
            });
        },
        // 处理获取到流之后的下载逻辑
        handleDownload (blob, fileName) {
            // webkix兼容性处理
            const URL = window.URL || window.webkitURL;
            const a = document.createElement('a');
            a.href = URL.createObjectURL(blob);
            // console.log(a.href);
            a.download = fileName;
            document.body.appendChild(a);
            a.click();
            // 在dom中移除a标签
            document.body.removeChild(a);
            // 在内存中移除URL对象
            window.URL.revokeObjectURL(URL);
        },

        // 通过接口查name
        getCategoryCode2Name (categoryCode, dictCode) {
            return new Promise(resolve => {
                this.$ajax.getCategoryCode2Name({categoryCode, dictCode}).then(res => {
                    if (res.code === 200) {
                        resolve(res.data);
                    } else {
                        this.$message.error(res.msg);
                        resolve(dictCode);
                    }
                }).catch(() => {
                    this.$message.error('网络异常，请稍后重试');
                    resolve(dictCode);
                });
            });
        },
    }
};
